@@ -4,9 +4,6 @@
 tween_colour <- function(data, n, ease = 'linear') {
   data <- as.list(data)
   prepData <- prepareTween(data, n, ease)
-  if (anyNA(suppressWarnings(col2rgb(unlist(prepData$data))))) {
-    stop('all elements in data must be convertible to colour')
-  }
   tweendata <- do.call(interpolate_colour_state, prepData)
   unname(split(tweendata,
                rep(seq_along(data[[1]]), length.out = length(tweendata))))
@@ -24,9 +21,6 @@ tween_colour_t <- function(data, n, ease = 'linear') {
     data <- list(data)
   }
   prepData <- prepareTweenTranspose(data, n, ease)
-  if (anyNA(suppressWarnings(col2rgb(unlist(prepData$data))))) {
-    stop('all elements in data must be convertible to colour')
-  }
   tweendata <- do.call(interpolate_colour_state, prepData)
   unname(split(tweendata,
                rep(seq_along(data), rep(n, length.out = length(data)))))
