@@ -9,7 +9,7 @@ test_that("tween_state works", {
   expect_true(all(tween$.phase[c(1:2, 9:10)] == 'raw'))
   expect_true(all(tween$.phase[c(3:8)] == 'transition'))
   expect_true(all(tween$.id == rep(1:2, 5)))
-  expect_equal(tween$col[5], '#767676FF')
+  expect_equal(tween$col[5], '#777777')
   expect_equal(tween$x[7], 8.5)
   expect_equal(tween$type[4:5], c('b', 'a'))
 })
@@ -32,6 +32,6 @@ test_that("enter/exit works", {
   })
   expect_equal(nrow(tween), 9)
   expect_true(all(tween$.phase[c(4,6,8)] == 'exit'))
-  expect_equal(tween$col[8], '#BA1808FF')
+  expect_equal(tween$col[8], '#BB1909')
   expect_equal(tween$x[8], 0.5)
 })
